/**

    Copyright 2007 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package it.eng.console.hibernate.pagination;

import it.eng.console.util.Constants;
import it.eng.spago.tracing.TracerSingleton;

import java.util.List;

import org.hibernate.HibernateException;
import org.hibernate.ScrollableResults;

/**
 * This class is taken by Hibernate wiki.
 * 
 * This class provides pagination for displaying results from a large result set
 * over a number of pages (i.e. with a given number of results per page).
 * 
 * Taken from http://blog.hibernate.org/cgi-bin/blosxom.cgi/2004/08/14#fn.html.
 * 
 * @author Gavin King
 * @author Eric Broyles
 */
public abstract class AbstractPage {

	protected ScrollableResults scrollableResults;
	protected List results;
	protected int pageSize;
	protected int page;
	protected int totalResults = 0;

	/**
	 * Construct a new Page. Page numbers are zero-based, so the first page is
	 * page 0.
	 * 
	 * @param query
	 *            the Hibernate Query
	 * @param page
	 *            the page number (zero-based)
	 * @param pageSize
	 *            the number of results to display on the page
	 */

	public AbstractPage(int page, int pageSize) throws HibernateException {
		this.page = page;
		this.pageSize = pageSize;
	}

	public boolean isFirstPage() {
		return page == 0;
	}

	public boolean isLastPage() {
		return page >= getLastPageNumber();
	}

	public boolean hasNextPage() {
		if(results == null)
			results = getResults();
		return results.size() > pageSize;
	}

	public boolean hasPreviousPage() {
		return page > 0;
	}

	public int getLastPageNumber() {
		/*
		 * We use the Math.floor() method because page numbers are zero-based
		 * (i.e. the first page is page 0).
		 */
		double totalResults = new Integer(getTotalResults()).doubleValue();
		return new Double(Math.floor(totalResults / pageSize)).intValue();
	}

	public List getList() {
		/*
		 * Since we retrieved one more than the specified pageSize when the
		 * class was constructed, we now trim it down to the pageSize if a next
		 * page exists.
		 */
		if(results == null)
			results = getResults();
		return hasNextPage() ? results.subList(0, pageSize) : results;
	}

	public int getFirstResultNumber() {
		return page * pageSize + 1;
	}

	public int getLastResultNumber() {
		int fullPage = getFirstResultNumber() + pageSize - 1;
		return getTotalResults() < fullPage ? getTotalResults() : fullPage;
	}

	public int getNextPageNumber() {
		return page + 1;
	}

	public int getPreviousPageNumber() {
		return page - 1;
	}
	
	public int getTotalResults() {
		try {
			getScrollableResults().last();
			totalResults = getScrollableResults().getRowNumber();
			
		} catch (HibernateException he) {
			TracerSingleton.log(Constants.project, TracerSingleton.CRITICAL, "HibernatePage::getTotalResults: fallita getRowNumber()", he);
            throw he;
		}
		return totalResults;
	}

	protected ScrollableResults getScrollableResults() {
		return scrollableResults;
	}
	
	public void setPage(int page) {
		this.page = page;
	}
	
	protected abstract List getResults();
	
	
}

